//
//  main.c
//
//  DeskClockControl 0.1
//  Copyright(C)2011 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		Tool for "DeskClock 0.1"
//
//	Usage
//		Connect a digital small article "Desk Clock".
//		Start up this program.
//
//	Environment:
//		Xcode 3.2.5
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2011.01.17 v0.0  First cording
//    2011.01.21 v0.1  Commit
//

#import "DeskClockControl01AppDelegate.h"
#define MAXPORTPATHNAME 64
#include <unistd.h>
#include <time.h>
#include <stdio.h>

@implementation DeskClockControl01AppDelegate

void SendCommand(char *cmd, char *res, size_t sz, NSString *fname){
	char	p[MAXPORTPATHNAME];
	int		fd;
	char	buff[20];
	
	if ([fname length] > 0) {
		if ([fname getCString:p maxLength:MAXPORTPATHNAME encoding:NSASCIIStringEncoding] == NO) {
			NSLog(@"serial device name error: %@", fname);
		} else {
			// Open
			if ((fd = open(p, O_RDWR | O_NOCTTY)) < 0) {
				NSLog(@"serial device open error: %@", fname);
			} else {
				
				// Send command string
				write(fd, cmd, strlen(cmd));

				// Wait response code
				for(;;) {
					char	*p;
					int		n;
					int		l;
					char	c;
					
					p = buff;
					n = sizeof(buff)- 1;
					while((l = read(fd, &c, 1)) > 0) {
						if (c == '\0') {		// Microchip "Application Libraries v2009-07-10" putUSBUSART() safe
							continue;
						}
						if (c == '\n') {
							break;
						}
						*p++ = c;
						--n;
						if (n <= 0) {
							break;
						}
					}
					*p = '\0';
					
					if (memcmp(buff, cmd, 3)) {		// Skip echo back
						break;
					}
				}
				if (res != NULL) {
					strncpy(res, buff, sz);
				}
				close(fd);
			}
		}
	}
}

- (IBAction)SetTime:(id)sender
{
	time_t t;
	struct tm *tm;
	char buff[10];
	
	t = time(NULL);
	tm = localtime(&t);
	sprintf(buff, "st%02d%02d\n",tm->tm_hour, tm->tm_min);
	SendCommand(buff, NULL, 0, [PortDev stringValue]);
}

- (IBAction)Set24h:(id)sender
{
	SendCommand("hh24\n", NULL, 0, [PortDev stringValue]);
}

- (IBAction)Set12h:(id)sender
{
	SendCommand("hh12\n", NULL, 0, [PortDev stringValue]);
}

- (IBAction)SetDim:(id)sender
{
	char buff[10];
	int s;
	int p;

	s = [ DimmStart intValue ];
	p = [ DimmStop intValue ];
	sprintf(buff, "dh%d\n", s);
	SendCommand(buff, NULL, 0, [PortDev stringValue]);
	sprintf(buff, "dl%d\n", p);
	SendCommand(buff, NULL, 0, [PortDev stringValue]);
}

- (IBAction)SetAdjust:(id)sender
{
	char buff[20];
	char res[20];
	int a;
	unsigned short t;
	int d;
	
	a = [ AdjustVal intValue ];
	t = [ NowTick intValue ];
	d = ((long)a * (long)t) / (24L * 60L * 60L);
	sprintf(buff, "aj%d\n", d);
	SendCommand(buff, res, sizeof(res), [PortDev stringValue]);
	t = atoi(&res[2]);
	[NowTick setIntValue:t];
	[AdjustVal setIntValue:0];	
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
	// Insert code here to initialize your application 
	NSString				*v = @"/dev/";
	NSDirectoryEnumerator	*d;
	NSString				*f;
	char					res[10];
	unsigned short			t;
	
	// Get device name
	d = [[NSFileManager defaultManager] enumeratorAtPath:v];
	[PortDev removeAllItems];
	while (f = [d nextObject]) {
		if ([f hasPrefix:@"cu.usbmodem#DC"]) {
			NSString	*p;
			
			p = [v stringByAppendingString:f];
			[PortDev addItemWithObjectValue:p];
		}
	}
	[PortDev selectItemAtIndex:0];

	// Get adjust tick
	SendCommand("aj0\n", res, sizeof(res), [PortDev stringValue]);
	t = atol(&res[3]);
	[NowTick setIntValue:t];
}

@end
